@@ -51,202 +51,235 @@ module type OrderedType = BatInterfaces.OrderedType
 (** Input signature of the functor {!Set.Make}. *)
 
 module type S =
-  sig
-    type elt
-    (** The type of the set elements. *)
+sig
+  type elt
+  (** The type of the set elements. *)
 
-    type t
-    (** The type of sets. *)
+  type t
+  (** The type of sets. *)
 
-    val empty: t
-    (** The empty set. *)
+  val empty: t
+  (** The empty set. *)
 
-    val is_empty: t -> bool
-    (** Test whether a set is empty or not. *)
+  val is_empty: t -> bool
+  (** Test whether a set is empty or not. *)
 
-    val singleton: elt -> t
-    (** [singleton x] returns the one-element set containing only [x]. *)
+  val singleton: elt -> t
+  (** [singleton x] returns the one-element set containing only [x]. *)
 
-    val mem: elt -> t -> bool
-    (** [mem x s] tests whether [x] belongs to the set [s]. *)
+  val mem: elt -> t -> bool
+  (** [mem x s] tests whether [x] belongs to the set [s]. *)
 
-    val add: elt -> t -> t
-    (** [add x s] returns a set containing all elements of [s],
-       plus [x]. If [x] was already in [s], [s] is returned unchanged. *)
+  val find : elt -> t -> elt
+  (** [find x s] returns the element in s that tests equal to [x] under its comparison function.
+      @raise Not_found if no element is equal
+  *)
 
-    val remove: elt -> t -> t
-    (** [remove x s] returns a set containing all elements of [s],
-       except [x]. If [x] was not in [s], [s] is returned unchanged. *)
+  val add: elt -> t -> t
+  (** [add x s] returns a set containing all elements of [s],
+      plus [x]. If [x] was already in [s], [s] is returned unchanged. *)
 
-    val union: t -> t -> t
-    (** Set union. *)
+  val remove: elt -> t -> t
+  (** [remove x s] returns a set containing all elements of [s],
+      except [x]. If [x] was not in [s], [s] is returned unchanged. *)
 
-    val inter: t -> t -> t
-    (** Set intersection. *)
+  val union: t -> t -> t
+  (** Set union. *)
 
-    val diff: t -> t -> t
-    (** Set difference. *)
+  val inter: t -> t -> t
+  (** Set intersection. *)
 
-    val sym_diff: t -> t -> t
-    (** [sym_diff s t] returns the set of all elements in [s] or [t]
+  val diff: t -> t -> t
+  (** Set difference. *)
+
+  val sym_diff: t -> t -> t
+  (** [sym_diff s t] returns the set of all elements in [s] or [t]
 	but not both.  This is the same as [diff (union s t) (inter s
 	t)]. *)
 
-    val compare: t -> t -> int
-    (** Total ordering between sets. Can be used as the ordering function
+  val compare: t -> t -> int
+  (** Total ordering between sets. Can be used as the ordering function
 	for doing sets of sets. *)
 
-    val equal: t -> t -> bool
-    (** [equal s1 s2] tests whether the sets [s1] and [s2] are
-       equal, that is, contain equal elements. *)
+  val equal: t -> t -> bool
+  (** [equal s1 s2] tests whether the sets [s1] and [s2] are
+      equal, that is, contain equal elements. *)
 
-    val subset: t -> t -> bool
-    (** [subset s1 s2] tests whether the set [s1] is a subset of
-       the set [s2]. *)
+  val subset: t -> t -> bool
+  (** [subset s1 s2] tests whether the set [s1] is a subset of
+      the set [s2]. *)
 
-    val disjoint: t -> t -> bool
-    (** [disjoint s1 s2] tests whether the sets [s1] and [s2] contain no shared
-        elements. (i.e. [inter s1 s2] is empty.) *)
+  val disjoint: t -> t -> bool
+  (** [disjoint s1 s2] tests whether the sets [s1] and [s2] contain no shared
+      elements. (i.e. [inter s1 s2] is empty.) *)
 
-    val compare_subset: t -> t -> int
-    (** Partial ordering between sets as generated by [subset] *)
+  val compare_subset: t -> t -> int
+  (** Partial ordering between sets as generated by [subset] *)
 
-    val iter: (elt -> unit) -> t -> unit
-    (** [iter f s] applies [f] in turn to all elements of [s].
-       The elements of [s] are presented to [f] in increasing order
-       with respect to the ordering over the type of the elements. *)
+  val iter: (elt -> unit) -> t -> unit
+  (** [iter f s] applies [f] in turn to all elements of [s].
+      The elements of [s] are presented to [f] in increasing order
+      with respect to the ordering over the type of the elements. *)
 
-    val map: (elt -> elt) -> t -> t
-      (** [map f x] creates a new set with elements [f a0],
+  val map: (elt -> elt) -> t -> t
+  (** [map f x] creates a new set with elements [f a0],
 	  [f a1]... [f aN], where [a0],[a1]..[aN] are the
 	  values contained in [x]*)
 
-    val filter: (elt -> bool) -> t -> t
-    (** [filter p s] returns the set of all elements in [s]
-       that satisfy predicate [p]. *)
+  val filter: (elt -> bool) -> t -> t
+  (** [filter p s] returns the set of all elements in [s]
+      that satisfy predicate [p]. *)
 
-    val filter_map: (elt -> elt option) -> t -> t
-      (** [filter_map f m] combines the features of [filter] and
+  val filter_map: (elt -> elt option) -> t -> t
+  (** [filter_map f m] combines the features of [filter] and
 	  [map].  It calls calls [f a0], [f a1], [f aN] where [a0],[a1]..[aN]
 	  are the elements of [m] and returns the set of pairs [bi]
 	  such as [f ai = Some bi] (when [f] returns [None], the
 	  corresponding element of [m] is discarded). *)
 
-    val fold: (elt -> 'a -> 'a) -> t -> 'a -> 'a
-      (** [fold f s a] computes [(f xN ... (f x1 (f x0 a))...)],
+  val fold: (elt -> 'a -> 'a) -> t -> 'a -> 'a
+  (** [fold f s a] computes [(f xN ... (f x1 (f x0 a))...)],
 	  where [x0],[x1]..[xN] are the elements of [s], in increasing order. *)
 
-    val for_all: (elt -> bool) -> t -> bool
-    (** [for_all p s] checks if all elements of the set
-       satisfy the predicate [p]. *)
+  val for_all: (elt -> bool) -> t -> bool
+  (** [for_all p s] checks if all elements of the set
+      satisfy the predicate [p]. *)
 
-    val exists: (elt -> bool) -> t -> bool
-    (** [exists p s] checks if at least one element of
-       the set satisfies the predicate [p]. *)
+  val exists: (elt -> bool) -> t -> bool
+  (** [exists p s] checks if at least one element of
+      the set satisfies the predicate [p]. *)
 
-    val partition: (elt -> bool) -> t -> t * t
-    (** [partition p s] returns a pair of sets [(s1, s2)], where
-       [s1] is the set of all the elements of [s] that satisfy the
-       predicate [p], and [s2] is the set of all the elements of
-       [s] that do not satisfy [p]. *)
+  val partition: (elt -> bool) -> t -> t * t
+  (** [partition p s] returns a pair of sets [(s1, s2)], where
+      [s1] is the set of all the elements of [s] that satisfy the
+      predicate [p], and [s2] is the set of all the elements of
+      [s] that do not satisfy [p]. *)
 
-    val split: elt -> t -> t * bool * t
-    (** [split x s] returns a triple [(l, present, r)], where
-          [l] is the set of elements of [s] that are
-          strictly less than [x];
-          [r] is the set of elements of [s] that are
-          strictly greater than [x];
-          [present] is [false] if [s] contains no element equal to [x],
-          or [true] if [s] contains an element equal to [x]. *)
+  val split: elt -> t -> t * bool * t
+  (** [split x s] returns a triple [(l, present, r)], where
+        [l] is the set of elements of [s] that are
+        strictly less than [x];
+        [r] is the set of elements of [s] that are
+        strictly greater than [x];
+        [present] is [false] if [s] contains no element equal to [x],
+        or [true] if [s] contains an element equal to [x]. *)
 
-    val cardinal: t -> int
-    (** Return the number of elements of a set. *)
+  val split_opt: elt -> t -> t * elt option * t
+  (** [split_opt x s] returns a triple [(l, maybe_v, r)], where
+        [l] is the set of elements of [s] that are
+        strictly less than [x];
+        [r] is the set of elements of [s] that are
+        strictly greater than [x];
+        [maybe_v] is [None] if [s] contains no element equal to [x],
+        or [Some v] if [s] contains an element [v] that compares equal to [x].
+      @since 2.2.0
+  *)
 
-    val elements: t -> elt list
-    (** Return the list of all elements of the given set.
-       The returned list is sorted in increasing order with respect
-       to the ordering [Ord.compare], where [Ord] is the argument
-       given to {!Set.Make}. *)
+  val split_lt: elt -> t -> t * t
+  (** [split_lt x s] returns a pair of sets [(l, r)], such that
+      [l] is the subset of [s] with elements < [x];
+      [r] is the subset of [s] with elements >= [x].
+      @since 2.2.0 *)
 
-    val min_elt: t -> elt
-    (** Return the smallest element of the given set
-       (with respect to the [Ord.compare] ordering).
+  val split_le: elt -> t -> t * t
+  (** [split_le x s] returns a pair of sets [(l, r)], such that
+      [l] is the subset of [s] with elements <= [x];
+      [r] is the subset of [s] with elements > [x].
+      @since 2.2.0 *)
+
+  val cardinal: t -> int
+  (** Return the number of elements of a set. *)
+
+  val elements: t -> elt list
+  (** Return the list of all elements of the given set.
+      The returned list is sorted in increasing order with respect
+      to the ordering [Ord.compare], where [Ord] is the argument
+      given to {!Set.Make}. *)
+
+  val to_list: t -> elt list
+  (** Alias for [elements].
+      @since 2.2.0 *)
+
+  val min_elt: t -> elt
+  (** Return the smallest element of the given set
+      (with respect to the [Ord.compare] ordering).
 
 	@raise Not_found if the set is empty. *)
 
-    val max_elt: t -> elt
-    (** Same as {!Set.S.min_elt}, but returns the largest element of the
-       given set. *)
+  val max_elt: t -> elt
+  (** Same as {!Set.S.min_elt}, but returns the largest element of the
+      given set. *)
 
-    val choose: t -> elt
-    (** Return one element of the given set, or raise [Not_found] if
-       the set is empty. Which element is chosen is unspecified,
-       but equal elements will be chosen for equal sets. *)
+  val choose: t -> elt
+  (** Return one element of the given set, or raise [Not_found] if
+      the set is empty. Which element is chosen is unspecified,
+      but equal elements will be chosen for equal sets. *)
 
-    val pop : t -> elt * t
-    (** returns one element of the set and the set without that element.
-        @raise Not_found if given an empty set *)
+  val pop : t -> elt * t
+  (** returns one element of the set and the set without that element.
+      @raise Not_found if given an empty set *)
 
-    val enum: t -> elt BatEnum.t
-      (** Return an enumeration of all elements of the given set.
+  val enum: t -> elt BatEnum.t
+  (** Return an enumeration of all elements of the given set.
 	  The returned enumeration is sorted in increasing order with respect
 	  to the ordering [Ord.compare], where [Ord] is the argument
 	  given to {!Set.Make}. *)
 
-    val backwards: t -> elt BatEnum.t
-      (** Return an enumeration of all elements of the given set.
+  val backwards: t -> elt BatEnum.t
+  (** Return an enumeration of all elements of the given set.
 	  The returned enumeration is sorted in decreasing order with respect
 	  to the ordering [Ord.compare], where [Ord] is the argument
 	  given to {!Set.Make}. *)
 
-    val of_enum: elt BatEnum.t -> t
+  val of_enum: elt BatEnum.t -> t
 
 
-    (** {6 Boilerplate code}*)
+  (** {6 Boilerplate code}*)
 
-    (** {7 Printing}*)
+  (** {7 Printing}*)
 
-    val print :  ?first:string -> ?last:string -> ?sep:string ->
-      ('a BatInnerIO.output -> elt -> unit) ->
-      'a BatInnerIO.output -> t -> unit
+  val print :  ?first:string -> ?last:string -> ?sep:string ->
+    ('a BatInnerIO.output -> elt -> unit) ->
+    'a BatInnerIO.output -> t -> unit
 
 
-      (** {6 Override modules}*)
+  (** {6 Override modules}*)
 
-    (**
-       The following modules replace functions defined in {!Set} with functions
-       behaving slightly differently but having the same name. This is by design:
-       the functions meant to override the corresponding functions of {!Set}.
-    *)
+  (**
+     The following modules replace functions defined in {!Set} with functions
+     behaving slightly differently but having the same name. This is by design:
+     the functions meant to override the corresponding functions of {!Set}.
+  *)
 
-    (** Operations on {!Set} without exceptions.*)
-    module Exceptionless : sig
-      val min_elt: t -> elt option
-      val max_elt: t -> elt option
-      val choose:  t -> elt option
-    end
+  (** Operations on {!Set} without exceptions.*)
+  module Exceptionless : sig
+    val min_elt: t -> elt option
+    val max_elt: t -> elt option
+    val choose:  t -> elt option
+    val find: elt -> t -> elt option
+  end
 
 
-    (** Operations on {!Set} with labels.
+  (** Operations on {!Set} with labels.
 
 	This module overrides a number of functions of {!Set} by
 	functions in which some arguments require labels. These labels are
 	there to improve readability and safety and to let you change the
 	order of arguments to functions. In every case, the behavior of the
 	function is identical to that of the corresponding function of {!Set}.
-    *)
-    module Labels : sig
-      val iter : f:(elt -> unit) -> t -> unit
-      val fold : f:(elt -> 'a -> 'a) -> t -> init:'a -> 'a
-      val for_all : f:(elt -> bool) -> t -> bool
-      val exists : f:(elt -> bool) -> t -> bool
-      val map: f:(elt -> elt) -> t -> t
-      val filter : f:(elt -> bool) -> t -> t
-      val filter_map: f:(elt -> elt option) -> t -> t
-      val partition : f:(elt -> bool) -> t -> t * t
-    end
-
+  *)
+  module Labels : sig
+    val iter : f:(elt -> unit) -> t -> unit
+    val fold : f:(elt -> 'a -> 'a) -> t -> init:'a -> 'a
+    val for_all : f:(elt -> bool) -> t -> bool
+    val exists : f:(elt -> bool) -> t -> bool
+    val map: f:(elt -> elt) -> t -> t
+    val filter : f:(elt -> bool) -> t -> t
+    val filter_map: f:(elt -> elt option) -> t -> t
+    val partition : f:(elt -> bool) -> t -> t * t
   end
+
+end
 (** Output signature of the functor {!Set.Make}. *)
 
 (*
@@ -273,7 +306,7 @@ module Make (Ord : OrderedType) : S with type elt = Ord.t
 
 (** {6 Polymorphic sets}
 
-   The definitions below describe the polymorphic set interface.
+    The definitions below describe the polymorphic set interface.
 
     They are similar in functionality to the functorized {!Make}
     module, but the compiler cannot ensure that sets using different
@@ -282,9 +315,9 @@ module Make (Ord : OrderedType) : S with type elt = Ord.t
     programmer. If in doubt, you should rather use the {!Make}
     functor for additional safety.
 
-   @author Nicolas Cannasse
-   @author Markus Mottl
-   @author David Rajchenbach-Teller
+    @author Nicolas Cannasse
+    @author Markus Mottl
+    @author David Rajchenbach-Teller
 *)
 
 type 'a t
@@ -294,30 +327,37 @@ include BatEnum.Enumerable with type 'a enumerable = 'a t
 include BatInterfaces.Mappable with type 'a mappable = 'a t
 
 val empty: 'a t
-  (** The empty set, using [compare] as comparison function *)
+(** The empty set, using [compare] as comparison function *)
 
 val is_empty: 'a t -> bool
-  (** Test whether a set is empty or not. *)
+(** Test whether a set is empty or not. *)
 
 val singleton : 'a -> 'a t
 (** Creates a new set with the single given element in it. *)
 
 val mem: 'a -> 'a t -> bool
-  (** [mem x s] tests whether [x] belongs to the set [s]. *)
+(** [mem x s] tests whether [x] belongs to the set [s]. *)
+
+val find: 'a -> 'a t -> 'a
+(** [find x s] returns the set element that compares equal to [x].
+    @raise Not_found if no such element exists
+
+    @since 2.1
+*)
 
 val add: 'a -> 'a t -> 'a t
-  (** [add x s] returns a set containing all elements of [s],
-      plus [x]. If [x] was already in [s], [s] is returned unchanged. *)
+(** [add x s] returns a set containing all elements of [s],
+    plus [x]. If [x] was already in [s], [s] is returned unchanged. *)
 
 val remove: 'a -> 'a t -> 'a t
-  (** [remove x s] returns a set containing all elements of [s],
-      except [x]. If [x] was not in [s], [s] is returned unchanged. *)
+(** [remove x s] returns a set containing all elements of [s],
+    except [x]. If [x] was not in [s], [s] is returned unchanged. *)
 
 val union: 'a t -> 'a t -> 'a t
-  (** [union s t] returns the union of [s] and [t] - the set containing
-      all elements in either [s] and [t].  The returned set uses [t]'s
-      comparison function.  The current implementation works better for
-      small [s]. *)
+(** [union s t] returns the union of [s] and [t] - the set containing
+    all elements in either [s] and [t].  The returned set uses [t]'s
+    comparison function.  The current implementation works better for
+    small [s]. *)
 
 (* Set.Make uses intersect *)
 val intersect: 'a t -> 'a t -> 'a t
@@ -325,8 +365,8 @@ val intersect: 'a t -> 'a t -> 'a t
     both [s] and [t].  The returned set uses [s]'s comparison function. *)
 
 val diff: 'a t -> 'a t -> 'a t
-  (** [diff s t] returns the set of all elements in [s] but not in
-      [t]. The returned set uses [s]'s comparison function.*)
+(** [diff s t] returns the set of all elements in [s] but not in
+    [t]. The returned set uses [s]'s comparison function.*)
 
 val sym_diff: 'a t -> 'a t -> 'a t
 (** [sym_diff s t] returns the set of all elements in [s] or [t] but
@@ -350,18 +390,21 @@ val disjoint: 'a t -> 'a t -> bool
     shared elements. (i.e. [inter s1 s2] is empty.) *)
 
 val iter: ('a -> unit) -> 'a t -> unit
-  (** [iter f s] applies [f] in turn to all elements of [s].
-      The elements of [s] are presented to [f] in increasing order
-      with respect to the ordering over the type of the elements. *)
+(** [iter f s] applies [f] in turn to all elements of [s].
+    The elements of [s] are presented to [f] in increasing order
+    with respect to the ordering over the type of the elements. *)
 
-val map: ('a -> 'a) -> 'a t -> 'a t
-  (** [map f x] creates a new set with elements [f a0],
-      [f a1]... [f aN], where [a0], [a1], ..., [aN] are the
-      values contained in [x]
+val map: ('a -> 'b) -> 'a t -> 'b t
+(** [map f x] creates a new set with elements [f a0],
+    [f a1]... [f aN], where [a0], [a1], ..., [aN] are the
+    elements of [x].
 
-      The resulting map uses the polymorphic [compare] function to
-      order elements.
-  *)
+    This function places no restriction on [f]; it can map multiple
+    input values to the same output value, in which case the
+    resulting set will have smaller cardinality than the input.  [f]
+    does not need to be order preserving, although if it is, then
+    [Incubator.op_map] may be more efficient.
+*)
 
 val filter: ('a -> bool) -> 'a t -> 'a t
 (** [filter p s] returns the set of all elements in [s]
@@ -369,68 +412,99 @@ val filter: ('a -> bool) -> 'a t -> 'a t
 
 (* as under-specified as 'map' *)
 val filter_map: ('a -> 'b option) -> 'a t -> 'b t
-  (** [filter_map f m] combines the features of [filter] and
-      [map].  It calls calls [f a0], [f a1], [f aN] where [a0,a1..an]
-      are the elements of [m] and returns the set of pairs [bi]
-      such as [f ai = Some bi] (when [f] returns [None], the
-      corresponding element of [m] is discarded).
+(** [filter_map f m] combines the features of [filter] and
+    [map].  It calls calls [f a0], [f a1], [f aN] where [a0,a1..an]
+    are the elements of [m] and returns the set of pairs [bi]
+    such as [f ai = Some bi] (when [f] returns [None], the
+    corresponding element of [m] is discarded).
 
-      The resulting map uses the polymorphic [compare] function to
-      order elements.
-  *)
+    The resulting map uses the polymorphic [compare] function to
+    order elements.
+*)
 
 val fold: ('a -> 'b -> 'b) -> 'a t -> 'b -> 'b
 (** [fold f s a] computes [(f xN ... (f x1 (f x0 a))...)],
     where [x0,x1..xN] are the elements of [s], in increasing order. *)
 
 val exists: ('a -> bool) -> 'a t -> bool
-  (** [exists p s] checks if at least one element of
-      the set satisfies the predicate [p]. *)
+(** [exists p s] checks if at least one element of
+    the set satisfies the predicate [p]. *)
 
 val for_all : ('a -> bool) -> 'a t -> bool
 (** Returns whether the given predicate applies to all elements in the set *)
 
 val partition : ('a -> bool) -> 'a t -> 'a t * 'a t
-  (** returns two disjoint subsets, those that satisfy the given
-      predicate and those that don't *)
+(** returns two disjoint subsets, those that satisfy the given
+    predicate and those that don't *)
 
 val split : 'a -> 'a t -> 'a t * bool * 'a t
-  (** [split x s] returns a triple [(l, present, r)], where
+(** [split x s] returns a triple [(l, present, r)], where
+    [l] is the set of elements of [s] that are
+    strictly less than [x];
+    [r] is the set of elements of [s] that are
+    strictly greater than [x];
+    [present] is [false] if [s] contains no element equal to [x],
+    or [true] if [s] contains an element equal to [x]. *)
+
+val split_opt: 'a -> 'a t -> 'a t * 'a option * 'a t
+(** [split_opt x s] returns a triple [(l, maybe_v, r)], where
       [l] is the set of elements of [s] that are
       strictly less than [x];
       [r] is the set of elements of [s] that are
       strictly greater than [x];
-      [present] is [false] if [s] contains no element equal to [x],
-      or [true] if [s] contains an element equal to [x]. *)
+      [maybe_v] is [None] if [s] contains no element equal to [x],
+      or [Some v] if [s] contains an element [v] that compares equal to [x].
+    @since 2.2.0
+*)
+
+val split_lt: 'a -> 'a t -> 'a t * 'a t
+(** [split_lt x s] returns a pair of sets [(l, r)], such that
+    [l] is the subset of [s] with elements < [x];
+    [r] is the subset of [s] with elements >= [x].
+    @since 2.2.0 *)
+
+val split_le: 'a -> 'a t -> 'a t * 'a t
+(** [split_le x s] returns a pair of sets [(l, r)], such that
+    [l] is the subset of [s] with elements <= [x];
+    [r] is the subset of [s] with elements > [x].
+    @since 2.2.0 *)
 
 val cardinal: 'a t -> int
-  (** Return the number of elements of a set. *)
+(** Return the number of elements of a set. *)
 
 val elements: 'a t -> 'a list
 (** Return the list of all elements of the given set.
     The returned list is sorted in increasing order with respect
     to the ordering of the given set. *)
 
+val to_list: 'a t -> 'a list
+(** Alias for [elements].
+    @since 2.2.0 *)
+
 val min_elt : 'a t -> 'a
-  (** returns the smallest element of the set.
-      @raise Invalid_argument if given an empty set. *)
+(** returns the smallest element of the set.
+    @raise Invalid_argument if given an empty set. *)
 
 val max_elt : 'a t -> 'a
-  (** returns the largest element of the set.
-      @raise Invalid_argument if given an empty set.*)
+(** returns the largest element of the set.
+    @raise Invalid_argument if given an empty set.*)
 
 val choose : 'a t -> 'a
-  (** returns an arbitrary (but deterministic) element of the given set.
-      @raise Invalid_argument if given an empty set. *)
+(** returns an arbitrary (but deterministic) element of the given set.
+    @raise Invalid_argument if given an empty set. *)
 
 val pop : 'a t -> 'a * 'a t
-  (** returns one element of the set and the set without that element.
-      @raise Not_found if given an empty set *)
+(** returns one element of the set and the set without that element.
+    @raise Not_found if given an empty set *)
+
+val cartesian_product : 'a t -> 'b t -> ('a * 'b) t
+(** cartesian product of the two sets
+    @since 2.2.0 *)
 
 val enum: 'a t -> 'a BatEnum.t
-  (** Return an enumeration of all elements of the given set.
-      The returned enumeration is sorted in increasing order with respect
-      to the ordering of this set.*)
+(** Return an enumeration of all elements of the given set.
+    The returned enumeration is sorted in increasing order with respect
+    to the ordering of this set.*)
 
 val of_enum: 'a BatEnum.t -> 'a t
 
@@ -455,6 +529,21 @@ val print :  ?first:string -> ?last:string -> ?sep:string ->
   'a BatInnerIO.output -> 'c t -> unit
 
 
+(** {6 Incubator} *)
+module Incubator : sig
+
+  val op_map : ('a -> 'b) -> 'a t -> 'b t
+    (** Order Preserving map; as [map], but [f] must be order preserving;
+        i.e. if [a < b] then [f a < f b].  This allows the tree structure
+        to be maintained internally, resulting in O(n) work instead of O(n
+        log n).
+
+        @since 2.1
+    *)
+
+end
+
+
 module PSet : sig
   (** {6 Polymorphic sets}
 
@@ -471,7 +560,7 @@ module PSet : sig
       @author Nicolas Cannasse
       @author Markus Mottl
       @author David Rajchenbach-Teller
-   *)
+  *)
 
   type 'a t
   (** The type of sets. *)
@@ -551,7 +640,7 @@ module PSet : sig
 
       The resulting map uses the polymorphic [compare] function to
       order elements.
-   *)
+  *)
 
   val filter: ('a -> bool) -> 'a t -> 'a t
   (** [filter p s] returns the set of all elements in [s]
@@ -567,7 +656,7 @@ module PSet : sig
 
       The resulting map uses the polymorphic [compare] function to
       order elements.
-   *)
+  *)
 
   val fold: ('a -> 'b -> 'b) -> 'a t -> 'b -> 'b
   (** [fold f s a] computes [(f xN ... (f x1 (f x0 a))...)],
@@ -593,6 +682,26 @@ module PSet : sig
       [present] is [false] if [s] contains no element equal to [x],
       or [true] if [s] contains an element equal to [x]. *)
 
+  val split_opt: 'a -> 'a t -> 'a t * 'a option * 'a t
+  (** [split_opt x s] returns a triple [(l, maybe_v, r)], where
+        [l] is the set of elements of [s] that are
+        strictly less than [x];
+        [r] is the set of elements of [s] that are
+        strictly greater than [x];
+        [maybe_v] is [None] if [s] contains no element equal to [x],
+        or [Some v] if [s] contains an element [v] that compares equal to [x].
+  *)
+
+  val split_lt: 'a -> 'a t -> 'a t * 'a t
+  (** [split_lt x s] returns a pair of sets [(l, r)], such that
+      [l] is the subset of [s] with elements < [x];
+      [r] is the subset of [s] with elements >= [x]. *)
+
+  val split_le: 'a -> 'a t -> 'a t * 'a t
+  (** [split_le x s] returns a pair of sets [(l, r)], such that
+      [l] is the subset of [s] with elements <= [x];
+      [r] is the subset of [s] with elements > [x]. *)
+
   val cardinal: 'a t -> int
   (** Return the number of elements of a set. *)
 
@@ -601,6 +710,9 @@ module PSet : sig
       The returned list is sorted in increasing order with respect
       to the ordering of the given set. *)
 
+  val to_list: 'a t -> 'a list
+  (** Alias for [elements]. *)
+
   val min_elt : 'a t -> 'a
   (** returns the smallest element of the set.
       @raise Invalid_argument if given an empty set. *)
